@@ -207,14 +207,20 @@
 		var defer = $.Deferred();
 		if (this.roomCollection && OC.getCurrentUser().uid) {
 			this.roomCollection.fetch({
-				success: function(data) {
-					defer.resolve(data);
+				success: function(roomCollection) {
+					defer.resolve(roomCollection);
+				},
+				error: function(roomCollection, response) {
+					defer.reject(roomCollection, response);
 				}
 			});
 		} else if (this.room) {
 			this.room.fetch({
-				success: function(data) {
-					defer.resolve(data);
+				success: function(room) {
+					defer.resolve(room);
+				},
+				error: function(room, response) {
+					defer.reject(room, response);
 				}
 			});
 		} else {
@@ -667,6 +673,9 @@
 			error: function (jqXHR, textStatus/*, errorThrown*/) {
 				if (jqXHR.status === 0 && textStatus === 'abort') {
 					// Request has been aborted. Ignore.
+				} else if (jqXHR.status === 404 || jqXHR.status === 403) {
+					console.log('Stop pulling messages because room does not exist or is not accessible');
+					this._trigger('pullMessagesStoppedOnFail');
 				} else if (this.currentRoomToken) {
 					if (this.pullMessagesFails >= 3) {
 						console.log('Stop pulling messages after repeated failures');
@@ -843,6 +852,7 @@
 			});
 		}
 		this.resumeId = null;
+		this.signalingRoomJoined = null;
 	};
 
 	OCA.Talk.Signaling.Standalone.prototype.disconnect = function() {
@@ -866,6 +876,8 @@
 			}
 
 			this._forceReconnect = true;
+			this.resumeId = null;
+			this.signalingRoomJoined = null;
 			return;
 		}
 
@@ -1008,8 +1020,10 @@
 		this._trigger("connect");
 		if (this.reconnected) {
 			// The list of rooms might have changed while we were not connected,
-			// so perform resync once.
-			this.internalSyncRooms();
+			// so perform resync once; force it to ensure that the resync is not
+			// waiting to retry a pending one failed due to a lack of
+			// connection.
+			this._forceInternalSyncRooms();
 			// Load any chat messages that might have been missed.
 			this._receiveChatMessages();
 		}
@@ -1057,10 +1071,13 @@
 		}.bind(this));
 	};
 
-	OCA.Talk.Signaling.Standalone.prototype.joinCall = function(token) {
+	OCA.Talk.Signaling.Standalone.prototype.joinCall = function(token, flags) {
 		if (this.signalingRoomJoined !== token) {
 			console.log("Not joined room yet, not joining call", token);
-			this.pendingJoinCall = token;
+			this.pendingJoinCall = {
+				token: token,
+				flags: flags
+			};
 			return;
 		}
 
@@ -1080,8 +1097,8 @@
 	OCA.Talk.Signaling.Standalone.prototype.joinResponseReceived = function(data, token) {
 		console.log("Joined", data, token);
 		this.signalingRoomJoined = token;
-		if (token === this.pendingJoinCall) {
-			this.joinCall(this.pendingJoinCall);
+		if (this.pendingJoinCall && token === this.pendingJoinCall.token) {
+			this.joinCall(this.pendingJoinCall.token, this.pendingJoinCall.flags);
 			this.pendingJoinCall = null;
 		}
 		if (this.roomCollection) {
@@ -1195,9 +1212,9 @@
 	};
 
 	OCA.Talk.Signaling.Standalone.prototype.syncRooms = function() {
-		if (this.pending_sync) {
+		if (this._pendingSyncRooms) {
 			// A sync request is already in progress, don't start another one.
-			return this.pending_sync;
+			return this._pendingSyncRooms;
 		}
 
 		// Never manually sync rooms, will be done based on notifications
@@ -1208,19 +1225,63 @@
 	};
 
 	OCA.Talk.Signaling.Standalone.prototype.internalSyncRooms = function() {
-		if (this.pending_sync) {
+		if (this._pendingSyncRooms) {
 			// A sync request is already in progress, don't start another one.
-			return this.pending_sync;
+			return this._pendingSyncRooms;
 		}
 
-		var defer = $.Deferred();
-		this.pending_sync = OCA.Talk.Signaling.Base.prototype.syncRooms.apply(this, arguments);
-		this.pending_sync.then(function(rooms) {
-			this.pending_sync = null;
+		this._pendingSyncRooms = $.Deferred();
+		this._waitTimeUntilSyncRetry = 1;
+		this._internalSyncRoomsWithRetry();
+		return this._pendingSyncRooms;
+	};
+
+	/**
+	 * Forces the synchronization of rooms.
+	 *
+	 * The rooms are synchronized immediately, even if the synchronization
+	 * failed before and there is a scheduled retry for later (which is
+	 * cancelled).
+	 *
+	 * Use sparingly, only when it is very likely that synchronizing again will
+	 * succeed despite having failed earlier (for example, after the Internet
+	 * connection has been restored).
+	 */
+	OCA.Talk.Signaling.Standalone.prototype._forceInternalSyncRooms = function() {
+		if (!this._pendingSyncRooms) {
+			return this.internalSyncRooms();
+		}
+
+		if (this._delayedInternalSyncRoomsWithRetry) {
+			clearTimeout(this._delayedInternalSyncRoomsWithRetry);
+			this._waitTimeUntilSyncRetry = 1;
+			this._internalSyncRoomsWithRetry();
+		} else {
+			// A synchronization is being performed right now, so there is
+			// nothing to do except for waiting.
+		}
+
+		return this._pendingSyncRooms;
+	};
+
+	OCA.Talk.Signaling.Standalone.prototype._internalSyncRoomsWithRetry = function() {
+		this._delayedInternalSyncRoomsWithRetry = null;
+
+		OCA.Talk.Signaling.Base.prototype.syncRooms.apply(this, arguments).then(function(rooms) {
+			// Remove _pendingSyncRooms before resolving it to make possible to
+			// sync again from handlers if needed.
+			var pendingSyncRooms = this._pendingSyncRooms;
+			this._pendingSyncRooms = null;
 			this.rooms = rooms;
-			defer.resolve(rooms);
+			pendingSyncRooms.resolve(rooms);
+		}.bind(this)).fail(function() {
+			this._delayedInternalSyncRoomsWithRetry = setTimeout(this._internalSyncRoomsWithRetry.bind(this), this._waitTimeUntilSyncRetry * 1000);
+
+			// Increase the wait time until retry to at most 8 seconds.
+			if (this._waitTimeUntilSyncRetry < 8) {
+				this._waitTimeUntilSyncRetry *= 2;
+			}
 		}.bind(this));
-		return defer;
 	};
 
 	OCA.Talk.Signaling.Standalone.prototype.processRoomListEvent = function(data) {
